# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from future.utils import iteritems
from past.builtins import basestring
from .configobj import Config
import click


class LabTestConfig(Config):
    default_config_files = [
        '.labtest.yml',
        '.labtest.yaml',
        'setup.cfg',
        'package.json',
    ]
    namespace = 'labtest'
    required_attrs = [
        'host',
        'app_name',
        'test_domain',
        'container_provider',
        'build_provider',
        'host_name_pattern',
        'environment',
        'docker_image_pattern',
    ]
    dependencies = {
        'build_provider': {
            'default': [
                'code_repo_url', 'app_build_image', 'app_build_command',
                'container_build_command', 'container_provider',
            ]
        }
    }

    def get_default_build_provider(self):
        """
        The default provider of the build
        """
        return 'default'

    def get_default_docker_image_pattern(self):
        """
        Return the default docker image name pattern
        """
        return '%(APP_NAME)s/%(INSTANCE_NAME)s:latest'

    def get_default_host_name_pattern(self):
        """
        Return the default host name pattern
        """
        return '%(APP_NAME)s-%(INSTANCE_NAME)s'

    def set_use_ssh_config(self, value):
        """
        Make sure the value is converted to a boolean
        """
        if isinstance(value, basestring):
            self._config['use_ssh_config'] = (value.lower() in ['1', 'true', 'yes', ])
        elif isinstance(value, int):
            self._config['use_ssh_config'] = (value == 1)
        elif isinstance(value, bool):
            self._config['use_ssh_config'] = value
        else:
            self._config['use_ssh_config'] = False

    def get_default_use_ssh_config(self):
        return False

    def set_environment(self, value):
        """
        Make sure the environment is a list
        """
        if isinstance(value, basestring):
            self._config['environment'] = value.split(',')
        elif isinstance(value, (list, tuple)):
            self._config['environment'] = list(value)  # convert tuples to list

    def get_default_environment(self):
        """
        Return an empty list as the default environment
        """
        return []

    def get_default_container_provider(self):
        """
        Where are the images stored by default?
        """
        return 'local'

    def get_default_container_build_command(self):
        return 'docker build -t $APP_NAME/$INSTANCE_NAME --build-arg RELEASE=$RELEASE --build-arg APP_NAME=$APP_NAME --build-arg BRANCH_NAME=$BRANCH_NAME --build-arg INSTANCE_NAME=$INSTANCE_NAME .'

    def get_default_app_name(self):
        """
        The default app_name is the name of the directory containing the .git directory
        """
        import os
        import subprocess
        out = subprocess.check_output(['git', 'rev-parse', '--show-toplevel'])
        dirname, name = os.path.split(out.strip())
        return name

    def validate_dependencies(self):
        """
        Make sure extra options are set, if necessary
        """
        config = self.config
        missing_attrs = []

        for option, dependency in iteritems(self.dependencies):
            for dep_option in dependency.get(config[option], []):
                if dep_option not in config:
                    default_func = getattr(self, 'get_default_{}'.format(dep_option), None)
                    if default_func:
                        setattr(self, dep_option, default_func())
                    else:
                        missing_attrs.append(dep_option)
        return missing_attrs


def get_config(filepath='', **kwargs):
    """
    Get the configuration based off all the ways you can pass it

    Can raise IOError if the filepath passed in doesn't exist

    Precedence:
        1. Command-line arguments
        2. Configuration file
    """
    config = LabTestConfig()
    if not filepath:
        config.parse_default_config()
    else:
        config.parse_file(filepath)

    for key, val in iteritems(kwargs):
        setattr(config, key, val)
    return config


@click.command()
@click.pass_context
def check_config(ctx):
    """
    Check the configuration and output any errors
    """
    ctx.obj.validate()
    click.echo(ctx.obj.validation_message())
    click.echo('')
    click.echo('Configuration:')
    for key, val in iteritems(ctx.obj.config):
        click.echo('  {}: {}'.format(key, val))
